/*
 *-----------------------------------------------------------------------------
 *
 * File:      adesto.c
 * Remarks:
 *            QSPI FLASH initialization loaders for various vendors
 *            The loaders are read by the booter in Manual mode
 *            in 1xSPI mode (ATM4 in 4x mode) and executed from RAM.
 *            If executed, the QSPI interface is fully configured for 4xSPI mode
 *            and the QSPI controller is set to auto mode. The loader ends
 *            by jumping to 0xF0000, the application main program.
 */

// include the default header
#include "config.h"
#include "sdk_defs.h"
#include "core_cm0.h"

#ifdef QSPI_ADESTO

/*--------------------------------------------------------------------------------*/
/*                                  ADESTO Related Functions                    */
/*--------------------------------------------------------------------------------*/

#define ADESTO_INST_WEL                 0x06
#define ADESTO_INST_QE                  0x35
#define ADESTO_INST_RDSR                0x05 // Opcode for STATUS READ command
#define ADESTO_INST_GBULK               0x98 // Opcode for Gang Block Unlock
#define ADESTO_INST_SERASE              0x20
#define ADESTO_INST_ERSUS               0xB0
#define ADESTO_INST_ERRES               0x30
#define ADESTO_INSTRUCTION_WRITE_ENABLE 0x06 //(write enable) or 0x50 (write enable for volatile). ??????
#define WRITE_STATUS_REGISTER_2         0x31

__inline void qspic_enable_quad_pads (void);
__inline void qspic_disable_quad_pads (void);
__inline void ADESTO_SetWriteEnable (void);
__inline void ADESTO_WaitEndOfWrite(void);
__inline void ADESTO_QPIEnable (void);
__inline void ADESTO_EnableQuadBit_status_register2(void);
__inline void ADESTO_AutoCfg_QuadIO (void);

uint8_t ADESTO_Init(void)
{
        // Disable quad pads. Memory are in single mode.
        qspic_disable_quad_pads();

        // Initialize the Status registers.
        // Disable Write Protection

        // Enable Quad mode
       ADESTO_QPIEnable();

        // IO3 hi-z, IO2 hi-z
        qspic_enable_quad_pads();

        // The Quad Enable bit (QE) of Status Register-2 must be set to enable the Fast read Quad I/O Instruction.
       ADESTO_EnableQuadBit_status_register2();

        // Fast Read Quad I/O instruction during the Auto mode
        ADESTO_AutoCfg_QuadIO();

        return(1);
}


// Enable the pads for quad operation
__inline void qspic_enable_quad_pads (void)
{
        QSPIC->QSPIC_CTRLMODE_REG &= ~(QSPIC_IO3_OEN | QSPIC_IO2_OEN);
}


// Disable the quad pads operation
__inline void qspic_disable_quad_pads (void)
{
        QSPIC->QSPIC_CTRLMODE_REG |= (QSPIC_IO3_OEN | QSPIC_IO2_OEN | QSPIC_IO3_DAT | QSPIC_IO2_DAT);
}


// Set WEL bit so write operations are valid
__inline void ADESTO_SetWriteEnable (void)
{
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = ADESTO_INST_WEL;
}


// Wait until the end of write/erase operation
__inline void ADESTO_WaitEndOfWrite(void)
{
	// Single Mode, CS Low
        QSPIC->QSPIC_CTRLBUS_REG = QSPIC_SET_SINGLE | QSPIC_EN_CS;

	// Read Status 1
	*((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = ADESTO_INST_RDSR;

	while ( QSPIC->QSPIC_READDATA_REG & 0x1);

	// CS High
	QSPIC->QSPIC_CTRLBUS_REG = QSPIC_DIS_CS;
}


// Enable Quad Mode
__inline void ADESTO_QPIEnable (void)
{
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = ADESTO_INST_QE;
}

__inline void ADESTO_EnableQuadBit_status_register2 (void)
{
        // Single Mode
        QSPIC->QSPIC_CTRLBUS_REG = QSPIC_SET_SINGLE;

        // Write Enable
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = ADESTO_INSTRUCTION_WRITE_ENABLE;

        // CS Low
        QSPIC->QSPIC_CTRLBUS_REG = QSPIC_EN_CS;


        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = WRITE_STATUS_REGISTER_2; // Write Status Register-2
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = 0x02; // Enable Quad SPI operation

        // CS High
        QSPIC->QSPIC_CTRLBUS_REG = QSPIC_DIS_CS;

        // Wait end of write
        ADESTO_WaitEndOfWrite();
}


// Fast Read Quad I/O instruction during the Auto mode
__inline void ADESTO_AutoCfg_QuadIO (void)
{
	//-------------------------------------------------------------------
	//                      Fast Read Quad I/O
	//-------------------------------------------------------------------
	// BustCmdA
	//-------------------------------------------------------------------
	// Command Value (IncBurst, Single) [7:0] : EB - Quad-Output Read
	// Command Value (WrapBurst)       [15:8] : 00
	// Extra byte                     [23:16] : 00 - Extra Byte After Address
	// Command Transmit Mode           [25:24] :  0 - Single SPI
	// Address Transmit Mode           [27:26] :  2 - Quad SPI
	// Extra Byte Transmit Mode        [29:28] :  2 - Quad SPI
	// Dummy Bytes Transmit Mode       [31:30] :  2 - Quad SPI
	//-------------------------------------------------------------------
	// BustCmdB
	//-------------------------------------------------------------------
        // Read Data Receive Mode           [1:0] :  2 - Quad
        // Extra Byte Enable                  [2] :  1 - Extra Byte Enabled
        // Extra Half Disable Out             [3] :  0 - Send the complete Extra Byte (if [2] ==1)
        // Num Of Dummy Bytes               [5:4] :  2 - 2 Dummy Bytes
        // Command mode                       [6] :  1 - Send Command Byte only in the first access.
        // Wrap mode                          [7] :  0 - Disable Wrap mode
        // Wrap Length                      [9:8] :  0 - Don't care
        // Wrap Size                      [11:10] :  0 - Don't care
        // CS High Min Num of CLKs        [14:12] :  0 - 0 CLKs

        QSPIC->QSPIC_BURSTCMDA_REG = 0xA8A000EB;
        QSPIC->QSPIC_BURSTCMDB_REG = 0x00000066;
        REG_SETF(QSPIC, QSPIC_CTRLMODE_REG, QSPIC_AUTO_MD, 1);
}
#endif
