/*
 *-----------------------------------------------------------------------------
 *
 * File:      eon.c
 * Remarks:     	
 *            QSPI FLASH initialization loaders for various vendors
 *            The loaders are read by the booter in Manual mode 
 *            in 1xSPI mode (ATM4 in 4x mode) and executed from RAM. 
 *            If executed, the QSPI interface is fully configured for 4xSPI mode
 *            and the QSPI controller is set to auto mode. The loader ends
 *            by jumping to 0xF0000, the application main program.
 */

// include the default header
#include "config.h"
#include "sdk_defs.h"
#include "core_cm0.h"

#ifdef QSPI_EON

__inline void qspic_disable_quad_pads (void);
__inline void qspic_enable_quad_pads (void);
__inline void qspic_reset_sequence (void);
__inline void EON_AutoCfg_QuadIO (void);

uint8 EON_Init(void)
{
        // Disable quad pads. Memory are in single mode.
        qspic_disable_quad_pads();

        // IO3 hi-z, IO2 hi-z
        qspic_enable_quad_pads();

        // RESET SEQUENCE
        qspic_reset_sequence();

        // Fast Read Quad I/O instruction during the Auto mode
        EON_AutoCfg_QuadIO();

	return(1);
}

// Disable the quad pads operation
__inline void qspic_disable_quad_pads (void)
{
        // Disable quad pads. Memory are in single mode.
        QSPIC->QSPIC_CTRLMODE_REG |= (QSPIC_IO3_OEN | QSPIC_IO2_OEN | QSPIC_IO3_DAT | QSPIC_IO2_DAT);
}

// Enable the pads for quad operation
__inline void qspic_enable_quad_pads (void)
{
        // Enable quad pads. IO3 hi-z, IO2 hi-z
        QSPIC->QSPIC_CTRLMODE_REG &= ~(QSPIC_IO3_OEN | QSPIC_IO2_OEN);
}

// RESET SEQUENCE
__inline void qspic_reset_sequence (void)
{
        // SW reset recovery
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = 0x66; // Reset-enable (RSTEN)
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = 0x99; // Reset-memory (RST)
}

// Fast Read Quad I/O instruction during the Auto mode
__inline void EON_AutoCfg_QuadIO (void) {
	//-------------------------------------------------------------------
	//                      Fast Read Quad I/O
	//-------------------------------------------------------------------
	// BustCmdA
	//-------------------------------------------------------------------
	// Command Value (IncBurst, Single) [7:0] : EB - Fast Read Quad I/O
	// Command Value (WrapBurst)       [15:8] : 00
	// Extra byte                     [23:16] : A5 - Extra Byte After Address
	// Command Transmit Mode           [25:24] :  0 - Quad
	// Address Transmit Mode           [27:26] :  2 - Quad
	// Extra Byte Transmit Mode        [29:28] :  2 - Quad
	// Dummy Bytes Transmit Mode       [31:30] :  2 - Quad
	//-------------------------------------------------------------------
	// BustCmdB
	//-------------------------------------------------------------------
	// Read Data Receive Mode           [1:0] :  2 - Quad
	// Extra Byte Enable                  [2] :  1 - Extra Byte Enabled
	// Extra Half Disable Out             [3] :  0 - Send the complete Extra Byte (if [2] ==1)
	// Num Of Dummy Bytes               [5:4] :  2 - 2 Dummy Bytes
	// Command mode                       [6] :  1 - Send Command Byte only in the first access.
	// Wrap mode                          [7] :  0 - Disable Wrap mode
	// Wrap Length                      [9:8] :  0 - Don't care
	// Wrap Size                      [11:10] :  0 - Don't care
	// CS High Min Num of CLKs        [14:12] :  0 - 0 CLKs

	QSPIC->QSPIC_BURSTCMDA_REG = 0xA8A500EB;
	QSPIC->QSPIC_BURSTCMDB_REG = 0x00000066;
	REG_SETF(QSPIC, QSPIC_CTRLMODE_REG, QSPIC_AUTO_MD, 1);
}

#endif


