/*
 *-----------------------------------------------------------------------------
 *
 * File:      macronix.c
 * Remarks:     	
 *            QSPI FLASH initialization loaders for various vendors
 *            The loaders are read by the booter in Manual mode 
 *            in 1xSPI mode (ATM4 in 4x mode) and executed from RAM. 
 *            If executed, the QSPI interface is fully configured for 4xSPI mode
 *            and the QSPI controller is set to auto mode. The loader ends
 *            by jumping to 0xF0000, the application main program.
 */

// include the default header
#include "config.h"
#include "sdk_defs.h"
#include "core_cm0.h"

#ifdef QSPI_MACRONIX

/*--------------------------------------------------------------------------------*/
/*                                  MACRONIX Related Functions                    */
/*--------------------------------------------------------------------------------*/

#define MX_INST_WEL   	0x06
#define MX_INST_QE    	0x35
#define MX_INST_RDSR  	0x05 // Opcode for STATUS READ command
#define MX_INST_GBULK 	0x98 // Opcode for Gang Block Unlock
#define MX_INST_SERASE	0x20
#define MX_INST_ERSUS	0xB0
#define MX_INST_ERRES	0x30

__inline void qspic_enable_quad_pads (void);
__inline void qspic_disable_quad_pads (void);
__inline void MX_SetWriteEnable (void);
__inline void MX_WaitEndOfWrite(void);
__inline void MX_QPIEnable (void);
__inline void MX_AutoCfg_QuadIO (void);

uint8_t MX_Init(void)
{
        // Disable quad pads. Memory are in single mode.
        qspic_disable_quad_pads();

        // Initialize the Status registers.
        // Disable Write Protection

        // Enable Quad mode
        MX_QPIEnable();

        // IO3 hi-z, IO2 hi-z
        qspic_enable_quad_pads();

        // Fast Read Quad I/O instruction during the Auto mode
        MX_AutoCfg_QuadIO();

        return(1);
}


// Enable the pads for quad operation
__inline void qspic_enable_quad_pads (void)
{
        QSPIC->QSPIC_CTRLMODE_REG &= ~(QSPIC_IO3_OEN | QSPIC_IO2_OEN);
}


// Disable the quad pads operation
__inline void qspic_disable_quad_pads (void)
{
        QSPIC->QSPIC_CTRLMODE_REG |= (QSPIC_IO3_OEN | QSPIC_IO2_OEN | QSPIC_IO3_DAT | QSPIC_IO2_DAT);
}


// Set WEL bit so write operations are valid
__inline void MX_SetWriteEnable (void)
{
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = MX_INST_WEL;
}


// Wait until the end of write/erase operation
__inline void MX_WaitEndOfWrite(void)
{
	// Single Mode, CS Low	
        QSPIC->QSPIC_CTRLBUS_REG = QSPIC_SET_SINGLE | QSPIC_EN_CS;
	
	// Read Status 1
	*((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = MX_INST_RDSR;
	
	while ( QSPIC->QSPIC_READDATA_REG & 0x1);
		
	// CS High
	QSPIC->QSPIC_CTRLBUS_REG = QSPIC_DIS_CS;
}


// Enable Quad Mode
__inline void MX_QPIEnable (void)
{
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = MX_INST_QE;
}

// Fast Read Quad I/O instruction during the Auto mode
__inline void MX_AutoCfg_QuadIO (void)
{
	//-------------------------------------------------------------------
	//                      Fast Read Quad I/O
	//-------------------------------------------------------------------
	// BustCmdA
	//-------------------------------------------------------------------
	// Command Value (IncBurst, Single) [7:0] : 0b - Quad-Output Read
	// Command Value (WrapBurst)       [15:8] : 00
	// Extra byte                     [23:16] : 00 - Extra Byte After Address
	// Command Transmit Mode           [25:24] :  2 - QSPI
	// Address Transmit Mode           [27:26] :  2 - QSPI
	// Extra Byte Transmit Mode        [29:28] :  2 - QSPI
	// Dummy Bytes Transmit Mode       [31:30] :  2 - QSPI
	//-------------------------------------------------------------------
	// BustCmdB
	//-------------------------------------------------------------------
	// Read Data Receive Mode           [1:0] :  2 - Quad
	// Extra Byte Enable                  [2] :  1 - Extra Byte Enabled
	// Extra Half Disable Out             [3] :  0 - Send the complete Extra Byte (if [2] ==1)
	// Num Of Dummy Bytes               [5:4] :  2 - 2 Dummy Bytes
	// Command mode                       [6] :  0 - Send Command Byte only in the first access.
	// Wrap mode                          [7] :  0 - Disable Wrap mode
	// Wrap Length                      [9:8] :  0 - Don't care
	// Wrap Size                      [11:10] :  0 - Don't care
	// CS High Min Num of CLKs        [14:12] :  1 - 1 CLKs	

        QSPIC->QSPIC_BURSTCMDA_REG = 0xaa00000b;
        QSPIC->QSPIC_BURSTCMDB_REG = 0x00001022;
}
#endif


