/*
 * File:      main.c
 * Remarks:
 *            QSPI FLASH initialization loaders for various vendors
 *            The loaders are read by the booter in Manual mode
 *            in 1xSPI mode (ATM4 in 4x mode) and executed from RAM.
 *            If executed, the QSPI interface is fully configured for 4xSPI mode
 *            and the QSPI controller is set to auto mode. The loader ends
 *            by jumping to 0xF0000, the application main program.
 *
 */
#include "config.h"
#include "sdk_defs.h"
#include "core_cm0.h"


//********* Memory MANUFACTURER IDs ************* //
#define WINBOND_ID              0xEF
#define MACRONIX_ID             0xC2
#define EON_ID                  0x1C
#define MICRON_ID               0x20
#define ADESTO_ID               0x1F
//*********************************************** //

#ifdef QSPI_WINBOND
uint8_t WND_Init(void);
#endif

#ifdef QSPI_MACRONIX
uint8_t MX_Init(void);
#endif

#ifdef QSPI_EON
uint8_t EON_Init(void);
#endif

#ifdef QSPI_MICRON
uint8_t MICRON_Init(void);
#endif

#ifdef QSPI_ADESTO
uint8_t ADESTO_Init(void);
#endif

void qspi_init(void)
{
	/* enable the QSPI clock */
        CRG_TOP->CLK_AMBA_REG |= 1 << REG_POS(CRG_TOP, CLK_AMBA_REG, QSPI_ENABLE);

	/* Set clock the divisions */
        REG_SETF(CRG_TOP, CLK_AMBA_REG, QSPI_DIV, 0);
        REG_SETF(CRG_TOP, CLK_AMBA_REG, HCLK_DIV, 0);
        REG_SETF(CRG_TOP, CLK_AMBA_REG, PCLK_DIV, 0);

	/* enable the QSPI pads */
	QSPIC->QSPIC_GP_REG = 0x1;
	REG_SET_BIT(CRG_TOP, SYS_CTRL_REG, PAD_LATCH_EN);  // open pad latches

	/* Power up peripherals power domain */
	REG_CLR_BIT(CRG_TOP, PMU_CTRL_REG, PERIPH_SLEEP);
	while (!REG_GETF(CRG_TOP, SYS_STAT_REG, PER_IS_UP)) {}

	/* set SINGLE mode for QSPIC */
	QSPIC->QSPIC_CTRLBUS_REG = QSPIC_DIS_CS | QSPIC_SET_SINGLE;// Disable serial access to device	+ SET QSPI IN SINGLE MODE
	QSPIC->QSPIC_CTRLMODE_REG = QSPIC_RXD_NEG | QSPIC_IO3_DAT | QSPIC_IO2_DAT | QSPIC_IO3_OEN | QSPIC_IO2_OEN;
}


int main(void)
{
        REG_SETF(CRG_TOP, CLK_16M_REG, RC16M_ENABLE, 1);
        REG_SETF(CRG_TOP, CLK_CTRL_REG, SYS_CLK_SEL, 1);

        GPREG->SET_FREEZE_REG = GPREG_SET_FREEZE_REG_FRZ_WDOG_Msk;
        WDOG->WATCHDOG_REG = 0xFF;
        REG_SET_BIT(CRG_TOP, SYS_CTRL_REG, DEBUGGER_ENABLE);
	while(((CRG_TOP->CLK_CTRL_REG) & RUNNING_AT_RC16M)==0);

	qspi_init();

#ifdef QSPI_WINBOND
	WND_Init();
#endif

#ifdef QSPI_MACRONIX
	MX_Init();
#endif

#ifdef QSPI_EON
	EON_Init();
#endif

#ifdef QSPI_MICRON
        MICRON_Init();
#endif

#ifdef QSPI_ADESTO
        ADESTO_Init();
#endif
	while(1);
}
