/*
 *-----------------------------------------------------------------------------
 *
 * File:      winbond.c
 * Remarks:     	
 *            QSPI FLASH initialization loaders for various vendors
 *            The loaders are read by the booter in Manual mode 
 *            in 1xSPI mode (ATM4 in 4x mode) and executed from RAM. 
 *            If executed, the QSPI interface is fully configured for 4xSPI mode
 *            and the QSPI controller is set to auto mode. The loader ends
 *            by jumping to 0xF0000, the application main program.
 */

// include the default header
#include "config.h"
#include "sdk_defs.h"
#include "core_cm0.h"

#ifdef QSPI_WINBOND
/*--------------------------------------------------------------------------------*/
/*                              Winbond Related Functions                         */
/*--------------------------------------------------------------------------------*/

#define WND_INST_MRSTQIO  0xff     // Mode bit Reset Quad I/O
#define WND_INST_MRSTDIO  0xffff   // Mode bit Reset Dual I/O
#define WND_INST_PDID     0xab     // Release Power Down / HPM / Device ID
#define WND_INST_RSTAT1   0x05     // Read Status Register 1
#define WND_INST_WEn      0x06     // Write Enable
#define WND_INST_WSTAT    0x01     // Write Status Register
#define WND_INST_HIGHP    0xa3     // High Performance Mode

#define WND_STATUS_QE      (0x1 << 1) // Quad Enable
#define WND_STATUS_BUSSY   (0x1 << 0) // Erase or Write i progress

__inline void qspic_enable_quad_pads (void);
__inline void qspic_disable_quad_pads (void);
__inline void WND_WaitEndOfWrite(void);
__inline void WND_WriteStatus (unsigned char w_status1, unsigned char w_status2);
__inline void WND_SetHighPerf (void);
__inline void WND_AutoCfg_QuadIO (void); 

uint8 WND_Init(void)
{
	// Disable quad pads. Memory are in single mode.
	qspic_disable_quad_pads();

	// Initialize the Status registers. 
	// Disable Write Protection and Enable Quad mode
	WND_WriteStatus(0x0,WND_STATUS_QE);
	
	// IO3 hi-z, IO2 hi-z 
	qspic_enable_quad_pads();
	
	// Enable high frequencies
	WND_SetHighPerf();
	
	// Fast Read Quad I/O instruction during the Auto mode
	WND_AutoCfg_QuadIO();
	
	return(1);
	
}

// Enable the pads for quad operation
__inline void qspic_enable_quad_pads (void) {
        QSPIC->QSPIC_CTRLMODE_REG &= ~(QSPIC_IO3_OEN | QSPIC_IO2_OEN);
}

// Disable the quad pads operation
__inline void qspic_disable_quad_pads (void){
        QSPIC->QSPIC_CTRLMODE_REG |= (QSPIC_IO3_OEN | QSPIC_IO2_OEN | QSPIC_IO3_DAT | QSPIC_IO2_DAT);
}


// Wait until the end of write/erase operation
__inline void WND_WaitEndOfWrite(void)
{
	// Single Mode, CS Low	
        QSPIC->QSPIC_CTRLBUS_REG = QSPIC_SET_SINGLE | QSPIC_EN_CS;
	
	// Read Status 1
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = WND_INST_RSTAT1;
	
	while ( *((volatile uint8_t *) &(QSPIC->QSPIC_READDATA_REG)) & WND_STATUS_BUSSY);
		
	// CS High
	QSPIC->QSPIC_CTRLBUS_REG = QSPIC_DIS_CS;
}

// Write Status and Configuration Registers
__inline void WND_WriteStatus (unsigned char w_status1, unsigned char w_status2) {
	// Single Mode
        QSPIC->QSPIC_CTRLBUS_REG = QSPIC_SET_SINGLE;
	
	// Write Enable
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = WND_INST_WEn;
	
	// CS Low
	QSPIC->QSPIC_CTRLBUS_REG = QSPIC_EN_CS;
	
	// Write Status Registers
	*((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = WND_INST_WSTAT;
	*((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = w_status1;
	*((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = w_status2;
	
	// CS High
	QSPIC->QSPIC_CTRLBUS_REG = QSPIC_DIS_CS;

	// Wait end of write
	WND_WaitEndOfWrite();
}

// Set High Performance - To operate in high frequencies.
__inline void WND_SetHighPerf (void)
{
	// Single Mode
        QSPIC->QSPIC_CTRLBUS_REG = QSPIC_SET_SINGLE;
	
	// Write Enable
        *((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = WND_INST_WEn;
		
	// CS Low
	QSPIC->QSPIC_CTRLBUS_REG = QSPIC_EN_CS;
	
	// Write High Performance Mode
	*((volatile uint8_t *) &(QSPIC->QSPIC_WRITEDATA_REG)) = WND_INST_HIGHP;

	// 3 - Dummy Bytes
	*((volatile uint8_t *) &(QSPIC->QSPIC_DUMMYDATA_REG)) = 0;
	*((volatile uint8_t *) &(QSPIC->QSPIC_DUMMYDATA_REG)) = 0;

	// CS High
	QSPIC->QSPIC_CTRLBUS_REG = QSPIC_DIS_CS;
}

// Fast Read Quad I/O instruction during the Auto mode
__inline void WND_AutoCfg_QuadIO (void)
{
	//-------------------------------------------------------------------
	//                      Fast Read Quad I/O
	//-------------------------------------------------------------------
	// BustCmdA
	//-------------------------------------------------------------------
	// Command Value (IncBurst, Single) [7:0] : eb - Fast Read Quad I/O
	// Command Value (WrapBurst)       [15:8] : 00
	// Extra byte                     [23:16] : a0 - Extra Byte After Address
	// Command Transmit Mode           [25:24] :  0 - SPI
	// Address Transmit Mode           [27:26] :  2 - Quad
	// Extra Byte Transmit Mode        [29:28] :  2 - Quad
	// Dummy Bytes Transmit Mode       [31:30] :  2 - Quad
	//-------------------------------------------------------------------
	// BustCmdB
	//-------------------------------------------------------------------
	// Read Data Receive Mode           [1:0] :  2 - Quad
	// Extra Byte Enable                  [2] :  1 - Extra Byte Enabled
	// Extra Half Disable Out             [3] :  0 - Send the complete Extra Byte (if [2] ==1)
	// Num Of Dummy Bytes               [5:4] :  2 - 2 Dummy Bytes
	// Command mode                       [6] :  1 - Send Command Byte only in the first access.
	// Wrap mode                          [7] :  0 - Disable Wrap mode
	// Wrap Length                      [9:8] :  0 - Don't care
	// Wrap Size                      [11:10] :  0 - Don't care
	// CS High Min Num of CLKs        [14:12] :  0 - 0 CLKs	

        QSPIC->QSPIC_BURSTCMDA_REG = 0xa8a000eb;
        QSPIC->QSPIC_BURSTCMDB_REG = 0x00000066;
	REG_SETF(QSPIC, QSPIC_CTRLMODE_REG, QSPIC_AUTO_MD, 1);
}
#endif


